% This script runs [OGA22, Algorithm 2] on the example of apocalypse
% of [LKB21] (https://arxiv.org/abs/2107.03877v1).
%% Function Q
D = diag([1 0.5]);
Y_apo = diag([1 0]);
Q = @(Y) 0.5*norm(D*(Y-Y_apo),'fro')^2;
DD = diag([1 0.25]);
g_Q = @(Y) DD*(Y-Y_apo);
%% Function phi
phi = @(t) 0.25*(t*(t*(t*t-2)-4)-2);
g_phi = @(t) t*(t*t-1)-1;
x_opt = 1.32471795724475;
%% Function f
f = @(X) Q(X(1:2,1:2)) + phi(X(3,3));
g_f = @(X) blkdiag(g_Q(X(1:2,1:2)),g_phi(X(3,3)));
f_opt = -1.932257884495233;
%% Parameters
X0 = diag([2 1 0]);
r = 2;
a0 = 8/5;
b = 1/2;
c = 1/5;
Delta = 0.1;
epsilon = 1e-8;
i_max = 100;
%% P2GD and P2GDR algorithms
tic
[X_all_P2GD, f_all_P2GD, s_all_P2GD] = P2GDRinfo(f, g_f, X0, r, a0, b, c, 0, epsilon, i_max);
time_P2GD = toc;
tic
[X_all, f_all, s_all] = P2GDRinfo(f, g_f, X0, r, a0, b, c, Delta, epsilon, i_max);
time = toc;
%% Iterates and distance to the minimizer
i_P2GD = length(f_all_P2GD);
X_diag_P2GD = zeros(i_P2GD,2);
DistToMin_P2GD = zeros(i_P2GD,1);
for j = 1:i_P2GD
    X_diag_P2GD(j,1) = X_all_P2GD{j}(1,1);
    X_diag_P2GD(j,2) = X_all_P2GD{j}(3,3);
    DistToMin_P2GD(j) = norm(X_diag_P2GD(j,:)-[1 x_opt]);
end
i = length(f_all);
X_diag = zeros(i,2);
DistToMin = zeros(i,1);
for j = 1:i
    X_diag(j,1) = X_all{j}(1,1);
    X_diag(j,2) = X_all{j}(3,3);
    DistToMin(j) = norm(X_diag(j,:)-[1 x_opt]);
end
%% Cost along the iterations
figure
box
hold on
plot(0:i_P2GD-1, log(f_all_P2GD-f_opt)/log(10), 'b.-', 'MarkerSize', 18);
plot(0:i-1, log(f_all-f_opt)/log(10), 'r.-', 'MarkerSize', 10);
xlabel('i');
ylabel('log_{10}(f(X_i)-f*)');
legend('P^2GD','P^2GDR');
set(gca, 'FontSize', 12);
%% Stationarity measure along the iterations
figure
box
hold on
plot(0:i_P2GD-1, log(s_all_P2GD)/log(10), 'b.-', 'MarkerSize', 18);
plot(0:i-1, log(s_all)/log(10), 'r.-', 'MarkerSize', 10);
xlabel('i');
ylabel('log_{10}(s_f(X_i))');
legend('P^2GD','P^2GDR');
set(gca, 'FontSize', 12);
%% Distance to the minimizer along the iterations
figure
box
hold on
plot(0:i_P2GD-1, log(DistToMin_P2GD)/log(10), 'b.-', 'MarkerSize', 18);
plot(0:i-1, log(DistToMin)/log(10), 'r.-', 'MarkerSize', 10);
xlabel('i');
ylabel('log_{10}(||X_i-X*||)');
legend('P^2GD','P^2GDR');
set(gca, 'FontSize', 12);